! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
!
! Module: temp_ssclib
!
! Authors:  Ian Stewart - SSC/LUX
!          
! Description: Contains routines taken from various modules of ssclib. Note that it is recommended to access these routines by linking to ssclib - copying them here should be viewed as a temporary measure.
! 
! Comments: TODO:
! 
! $Id: ssc_misc.f90,v 1.6 1998/05/27 13:45:39 gvacanti Exp $
!- 
module temp_ssclib

  use types

  implicit none 

  private 

  public isNaN			! ssc_misc
  public smallestTrueIndex	! array_utils

  interface isNaN
    module procedure isNaNSingleScal, isNaNSingleVec, isNaNSingleArr
    module procedure isNaNDoubleScal, isNaNDoubleVec, isNaNDoubleArr
  end interface

contains 

  !---------------------------------------------------------------------
  logical function isNaNSingleScal(arg)
  !---------------------------------------------------------------------
  ! NaNs don't seem to return true for any comparison, hence the test
  ! below. NOTE however that I haven't checked the f95 standard to see if
  ! this is a mandatory feature or just happens to be true on solaris-8.

    implicit none

    real(single), intent(in) :: arg

    !-------------------------------------------------------------------
    ! Start

    if(.not.(arg > 0.0) .and. .not.(arg <= 0.0)) then
      isNaNSingleScal = .true.
    else
      isNaNSingleScal = .false.
    end if

  end function isNaNSingleScal

  !---------------------------------------------------------------------
  function isNaNSingleVec(arg)
  !---------------------------------------------------------------------
  ! NaNs don't seem to return true for any comparison, hence the test
  ! below. NOTE however that I haven't checked the f95 standard to see if
  ! this is a mandatory feature or just happens to be true on solaris-8.

    implicit none

    real(single), intent(in) :: arg(:)
    logical                  :: isNaNSingleVec(size(arg))

    !-------------------------------------------------------------------
    ! Start

    where(.not.(arg > 0.0) .and. .not.(arg <= 0.0))
      isNaNSingleVec = .true.
    elsewhere
      isNaNSingleVec = .false.
    end where

  end function isNaNSingleVec

  !---------------------------------------------------------------------
  function isNaNSingleArr(arg)
  !---------------------------------------------------------------------
  ! NaNs don't seem to return true for any comparison, hence the test
  ! below. NOTE however that I haven't checked the f95 standard to see if
  ! this is a mandatory feature or just happens to be true on solaris-8.

    implicit none

    real(single), intent(in) :: arg(:,:)
    logical                  :: isNaNSingleArr(size(arg,1),size(arg,2))

    !-------------------------------------------------------------------
    ! Start

    where(.not.(arg > 0.0) .and. .not.(arg <= 0.0))
      isNaNSingleArr = .true.
    elsewhere
      isNaNSingleArr = .false.
    end where

  end function isNaNSingleArr

  !---------------------------------------------------------------------
  logical function isNaNDoubleScal(arg)
  !---------------------------------------------------------------------
  ! NaNs don't seem to return true for any comparison, hence the test
  ! below. NOTE however that I haven't checked the f95 standard to see if
  ! this is a mandatory feature or just happens to be true on solaris-8.

    implicit none

    real(double), intent(in) :: arg

    !-------------------------------------------------------------------
    ! Start

    if(.not.(arg > 0.0) .and. .not.(arg <= 0.0)) then
      isNaNDoubleScal = .true.
    else
      isNaNDoubleScal = .false.
    end if

  end function isNaNDoubleScal

  !---------------------------------------------------------------------
  function isNaNDoubleVec(arg)
  !---------------------------------------------------------------------
  ! NaNs don't seem to return true for any comparison, hence the test
  ! below. NOTE however that I haven't checked the f95 standard to see if
  ! this is a mandatory feature or just happens to be true on solaris-8.

    implicit none

    real(double), intent(in) :: arg(:)
    logical                  :: isNaNDoubleVec(size(arg))

    !-------------------------------------------------------------------
    ! Start

    where(.not.(arg > 0.0) .and. .not.(arg <= 0.0))
      isNaNDoubleVec = .true.
    elsewhere
      isNaNDoubleVec = .false.
    end where

  end function isNaNDoubleVec

  !---------------------------------------------------------------------
  function isNaNDoubleArr(arg)
  !---------------------------------------------------------------------
  ! NaNs don't seem to return true for any comparison, hence the test
  ! below. NOTE however that I haven't checked the f95 standard to see if
  ! this is a mandatory feature or just happens to be true on solaris-8.

    implicit none

    real(double), intent(in) :: arg(:,:)
    logical                  :: isNaNDoubleArr(size(arg,1),size(arg,2))

    !-------------------------------------------------------------------
    ! Start

    where(.not.(arg > 0.0) .and. .not.(arg <= 0.0))
      isNaNDoubleArr = .true.
    elsewhere
      isNaNDoubleArr = .false.
    end where

  end function isNaNDoubleArr

  !---------------------------------------------------------------------
  integer function smallestTrueIndex(maskVector)
  !---------------------------------------------------------------------
  ! The functions smallestTrueIndex() and largestTrueIndex() are related
  ! to maskArrayLimits() but are much more basic. The function
  ! smallestTrueIndex() returns the index of the first TRUE value of the
  ! input logical vector maskVector(). The first index is assumed to be 1
  ! not 0.
  !
  ! If maskVector() contains no TRUE values, size(maskVector) + 1 is returned.

    implicit none

    logical, intent(in) :: maskVector(:)

    integer :: i

    !-------------------------------------------------------------------
    ! Start

    smallestTrueIndex = size(maskVector) + 1
    do i = 1, size(maskVector)
    if (maskVector(i)) then
        smallestTrueIndex = i
    exit
      end if
    end do

  end function smallestTrueIndex

  !---------------------------------------------------------------------
  subroutine template
  !---------------------------------------------------------------------

    implicit none

    !-------------------------------------------------------------------
    ! Start

  end subroutine template

end module temp_ssclib
